#!/bin/ksh
#
# File: apply_pending_on_demand.sh
# This script is used to run apply_pending.sh when a flag file is placed on the host. 
# Author: Doug Jackson  12/10/2010.


#--------------------------------------------------------------------
#- Clean Up
#--------------------------------------------------------------------
clean_up()
{
  if [ ! -z "`grep -i NOTE $APPLY_LOG`" ]
  then
    SUBJECT="NOTE: Apply Pending for $SID can not be completed at this time."
    SEND_EMAIL=YES
  else
    SUBJECT="Successfully Launched Apply Pending for: $SID $SCHEMA"
    SEND_EMAIL=NO
  fi

  if [ "$SEND_EMAIL" = "YES" ]
  then
    echo "" >>$APPLY_LOG
    echo "Notified:" >>$APPLY_LOG
    cat $EMAIL_LIST ${FLAG_FILE}_in_progress | grep @ | grep -v "^#" | sort | uniq >>$APPLY_LOG

    for EACH in `cat $EMAIL_LIST $FILE | grep @ | grep -v "^#" | sort | uniq`
    do
      mail -s "$SUBJECT" $EACH < $APPLY_LOG
    done
  fi

  rm -f ${FLAG_FILE} 2>/dev/null
  rm -f ${FLAG_FILE}_in_progress 2>/dev/null
}



#################################################################################################################
### MAIN PROGRAM
#################################################################################################################

# Source oracle's profile
. ~/.profile noup

SCRIPT_DIR=`dirname $0`
if [ "$SCRIPT_DIR" = "." ]
then
  SCRIPT_DIR=`pwd`
fi
cd $SCRIPT_DIR

HOST=`hostname | cut -d. -f1`

# Configuration
FLAG_DIR="/u01/flag_ftp"
EMAIL_LIST="$SCRIPT_DIR/backup_email.txt"
ORATAB=/etc/oratab

# Exit if there are no apply pending flags that are not being processed.
if [ ! -f $FLAG_DIR/flag_apply_pending_*.txt ]
then
  exit
fi

# Pickup the oldest flag_backup_ file
FLAG_FILE=`ls $FLAG_DIR/flag_apply_pending_*.txt | head -1`

APPLY_LOG=$SCRIPT_DIR/`basename $FLAG_FILE | cut -d. -f1`.log
> $APPLY_LOG

echo Flag Apply Pending on Demand - apply_pending_on_demand.sh >> $APPLY_LOG
COMPUTER_NAME=`grep COMPUTER_NAME $FLAG_FILE | cut -d= -f2`
USER_NAME=`grep USER_NAME $FLAG_FILE | cut -d= -f2`
ONLY_NEW=`grep ONLY_NEW $FLAG_FILE | cut -d= -f2`
echo "Apply pending scripts was requested by $USER_NAME on $COMPUTER_NAME." >>$APPLY_LOG
echo `date` >> $APPLY_LOG
echo Host: $HOST >> $APPLY_LOG

# Check for the sid
SID=`echo ${FLAG_FILE} | awk -F"_apply_pending_" '{print $2}' | cut -d_ -f1  | cut -d. -f1 | tr [a-z] [A-Z]`
SCHEMA=`echo ${FLAG_FILE} | awk -F"_apply_pending_" '{print $2"_"}' | cut -d_ -f2  | cut -d. -f1 | tr [a-z] [A-Z]`
if [ -z "`grep ^${SID}: /etc/oratab`" ]
then
  echo "ERROR: The database: $SID was not found on this host." >>$APPLY_LOG
  LOG=$APPLY_LOG
  rm $FLAG_FILE
  clean_up
  exit
fi
echo Database: $SID >>$APPLY_LOG
echo Schema: $SCHEMA >>$APPLY_LOG

# Rename Flag File to _in_progress 
mv $FLAG_FILE ${FLAG_FILE}_in_progress

# Check that the database is up. We don't want the backup.sh script to start the database.
ORACLE_SID=$SID
# Set the ORACLE_HOME, PATH, LIBPATH, etc
. oraenv 2>/dev/null
. set_base.sh
. set_lib.sh
. set_env.sh
STATUS=`sqlplus -s /nolog << ENDIT
set pause off head off pagesize 0 feedback off linesize 200
connect / as sysdba
connect bogus/bogus
exit
ENDIT`
if [ ! -z "`echo $STATUS | grep ORA-01090`"  ]
then
  STATUS="SHUTDOWN"
else
  if [ ! -z "`echo $STATUS | grep ORA-01033`" ]
  then
    STATUS="STARTUP"
  else
    if [ ! -z "`echo $STATUS | grep ORA-27101`" ]
    then
      STATUS="DOWN"
    else
      STATUS=`sqlplus -s /nolog << ENDIT
set pause off head off pagesize 0 feedback off linesize 200
connect / as sysdba
select open_mode from v\\$database;
exit
ENDIT`
    fi
  fi
fi
echo Database Status: $STATUS >>$APPLY_LOG

if [ "$STATUS" = "SHUTDOWN" ] || [ "$STATUS" = "STARTUP" ] || [ "$STATUS" = "DOWN" ] 
then
  echo "NOTE: The $SID database is not up so scripts can not be applied at this time." >>$APPLY_LOG
  clean_up
  exit
fi

# Kick off apply_pending.sh 
if [ "$ONLY_NEW" = "Y" ]
then
  echo "Running: $SCRIPT_DIR/apply_pending.sh $SID -new" >>$APPLY_LOG
  $SCRIPT_DIR/apply_pending.sh $SID $SCHEMA -new 
else
  echo "Running: $SCRIPT_DIR/apply_pending.sh $SID" >>$APPLY_LOG
  $SCRIPT_DIR/apply_pending.sh $SID $SCHEMA 
fi

echo "" >>$APPLY_LOG

clean_up
exit
